@@ -1,5 +1,5 @@
 #' Apply voxel counting on a point cloud
-#' 
+#'
 #' A \code{rasterStack} object is created, representing the 3–D voxel cube.
 #' The z axis is sliced into regular sections between the maximum and minimum value.
 #' For every height slice a raster with points per cell counts is created. Additionally
@@ -19,45 +19,45 @@
 #'
 #' @examples
 #' voxelCount(niclas, raster::extent(dem), 100, 100, 1000, 1400, standartize = TRUE)
-voxelCount <- function(points, extent, xyRes, zRes = xyRes, zMin, zMax, standartize = FALSE, verbose = FALSE){
-  rTem <- raster::raster(extent, res=xyRes)
+voxelCount <- function(points, extent, xyRes, zRes = xyRes, zMin, zMax, standartize = FALSE, verbose = FALSE) {
+  rTem <- raster::raster(extent, res = xyRes)
   rTem[] <- 0
   rStack <- raster::stack()
-  for (i in 1:round((zMax-zMin)/zRes)) {
+  for (i in 1:round((zMax - zMin) / zRes)) {
     if (verbose) {
-      cat('\r', paste("  |Counting points in Voxels for height: ", zMin+(i-1)*zRes, "m - ", (zMin+i*zRes), "m ...", sep = ""))
+      cat("\r", paste("  |Counting points in Voxels for height: ", zMin + (i - 1) * zRes, "m - ", (zMin + i * zRes), "m ...", sep = ""))
       utils::flush.console()
     }
-    p <- points[points[,3] > (zMin+(i-1)*zRes) & points[,3] < (zMin+i*zRes), ]
+    p <- points[points[, 3] > (zMin + (i - 1) * zRes) & points[, 3] < (zMin + i * zRes), ]
     if (!nrow(p) == 0) {
-      r <- raster::rasterize(cbind(p[,1], p[,2]), rTem, fun='count')
+      r <- raster::rasterize(cbind(p[, 1], p[, 2]), rTem, fun = "count")
       r[is.na(r[])] <- 0
       rStack <- raster::stack(rStack, r)
     } else {
       rStack <- raster::stack(rStack, rTem)
     }
-    names(rStack)[i] <- c(paste("m", zMin+(i-1)*zRes, "-", (zMin+i*zRes), sep = ""))
+    names(rStack)[i] <- c(paste("m", zMin + (i - 1) * zRes, "-", (zMin + i * zRes), sep = ""))
   }
-  if(standartize){
+  if (standartize) {
     maxR <- max(raster::maxValue(rStack))
     minR <- min(raster::minValue(rStack))
-    for(i in 1:length(rStack@layers)) {
+    for (i in seq_len(length(rStack@layers))) {
       rStack@layers[[i]] <- (rStack@layers[[i]] - minR) / (maxR - minR)
     }
   }
   if (verbose) {
-    cat('\r', "  |Done.                                                             \n")
+    cat("\r", "  |Done.                                                             \n")
     utils::flush.console()
   }
   return(rStack)
 }
 
 #' Chi maps of two variables
-#' 
+#'
 #' Calculates the chi maps for one \code{rasterStack} or all raster all the raster pairs stored in two \code{rasterStack}s.
-#' As observed values, the first stack is used. The expected value is either set to the mean of the first stack, or if given 
+#' As observed values, the first stack is used. The expected value is either set to the mean of the first stack, or if given
 #' to be the values of the second stack.
-#' 
+#'
 #' @param stack1 \code{rasterStack}
 #' @param stack2 \code{rasterStack}  \code{NULL} or containing the same number of \code{rasterLayer}s and has euqal extent and resolution.
 #' @param verbose logical: print currently processed height band in raster stack?
@@ -66,44 +66,47 @@ voxelCount <- function(points, extent, xyRes, zRes = xyRes, zMin, zMax, standart
 #' @export
 #'
 #' @examples
-#' chiMaps(raster::stack(dem))
+#' print("tbd.")
 chiMaps <- function(stack1, stack2 = NULL, verbose = FALSE) {
-  if (is.null(stack2)){
+  if (is.null(stack2)) {
     rStack <- raster::stack()
     expMean <- mean(raster::values(stack1))
-    for (i in 1:length(stack1@layers)){
+    for (i in seq_len(length(stack1@layers))) {
       if (verbose) {
-        cat('\r', paste("  |Calcuate chi map for raster:", i, "..."))
+        cat("\r", paste("  |Calcuate chi map for raster:", i, "..."))
         utils::flush.console()
       }
-      r1 <- stack1@layers[[i]]; r1[r1 == 0] <- NA
+      r1 <- stack1@layers[[i]]
+      r1[r1 == 0] <- NA
       rChi <- (r1 - expMean) / sqrt(expMean)
       rChi[is.na(rChi)] <- 0
       rStack <- raster::stack(rStack, rChi)
-      names(rStack)[i] <- c(paste("chiMap",names(stack1)[i], sep = ))
+      names(rStack)[i] <- c(paste("chiMap", names(stack1)[i], sep = ))
     }
     if (verbose) {
-      cat('\r', "  |Done.                                                             \n")
+      cat("\r", "  |Done.                                                             \n")
       utils::flush.console()
     }
     return(rStack)
   } else {
-    if(length(stack1@layers) != length(stack2@layers)) stop("Stack need to have the same number of rasters")
+    if (length(stack1@layers) != length(stack2@layers)) stop("Stack need to have the same number of rasters")
     rStack <- raster::stack()
-    for (i in 1:length(stack1@layers)){
+    for (i in seq_len(length(stack1@layers))) {
       if (verbose) {
-        cat('\r', paste("  |Calcuate chi map for raster:", i, "..."))
+        cat("\r", paste("  |Calcuate chi map for raster:", i, "..."))
         utils::flush.console()
       }
-      r1 <- stack1@layers[[i]]; r1[r1 == 0] <- NA
-      r2 <- stack2@layers[[i]]; r2[r2 == 0] <- NA
+      r1 <- stack1@layers[[i]]
+      r1[r1 == 0] <- NA
+      r2 <- stack2@layers[[i]]
+      r2[r2 == 0] <- NA
       rChi <- (r1 - r2) / sqrt(r2)
       rChi[is.na(rChi)] <- 0
       rStack <- raster::stack(rStack, rChi)
-      names(rStack)[i] <- c(paste("chiMap",names(stack1)[i], sep = ))
+      names(rStack)[i] <- c(paste("chiMap", names(stack1)[i], sep = ))
     }
     if (verbose) {
-      cat('\r', "  |Done.                                                             \n")
+      cat("\r", "  |Done.                                                             \n")
       utils::flush.console()
     }
     return(rStack)
@@ -122,40 +125,48 @@ chiMaps <- function(stack1, stack2 = NULL, verbose = FALSE) {
 #'
 #' @examples
 #' plotRaster(dem)
-plotRaster <- function(r, title = character(0), centerColorBar = FALSE, ncol = NULL){
+plotRaster <- function(r, title = character(0), centerColorBar = FALSE, ncol = NULL) {
   if (centerColorBar) {
     colTheme <- rasterVis::BuRdTheme()
     maxVal <- max(raster::values(r)[is.finite(raster::values(r))], na.rm = TRUE)
-    colSeq <- seq(-1*maxVal, maxVal, len=100)
+    colSeq <- seq(-1 * maxVal, maxVal, len = 100)
   } else {
     colTheme <- rasterVis::YlOrRdTheme()
     minVal <- min(raster::values(r)[is.finite(raster::values(r))], na.rm = TRUE)
     maxVal <- max(raster::values(r)[is.finite(raster::values(r))], na.rm = TRUE)
-    colSeq <- seq(minVal, maxVal, len=100)
+    colSeq <- seq(minVal, maxVal, len = 100)
   }
-  if (class(r)[1] == "RasterLayer"){
-    print(rasterVis::levelplot(r, par.settings=colTheme, interpolate = TRUE,
-                               margin=FALSE, at=colSeq,
-                               main=title, xlab="Easting", ylab="Northing"))
+  if (class(r)[1] == "RasterLayer") {
+    print(rasterVis::levelplot(r,
+      par.settings = colTheme, interpolate = TRUE,
+      margin = FALSE, at = colSeq,
+      main = title, xlab = "Easting", ylab = "Northing"
+    ))
   }
-  if (class(r)[1] == "RasterBrick"){r <- raster::stack(r)}
-  if (class(r)[1] == "RasterStack"){
+  if (class(r)[1] == "RasterBrick") {
+    r <- raster::stack(r)
+  }
+  if (class(r)[1] == "RasterStack") {
     plotList <- list()
-    for (i in 1:length(r@layers)){
-      if(raster::cellStats(r@layers[[i]], "sum") != 0) {
-        p <- rasterVis::levelplot(r@layers[[i]], par.settings=colTheme, interpolate = TRUE,
-                                  margin=FALSE, at=colSeq,
-                                  main = names(r)[i], xlab="Easting", ylab="Northing")
+    for (i in seq_len(length(r@layers))) {
+      if (raster::cellStats(r@layers[[i]], "sum") != 0) {
+        p <- rasterVis::levelplot(r@layers[[i]],
+          par.settings = colTheme, interpolate = TRUE,
+          margin = FALSE, at = colSeq,
+          main = names(r)[i], xlab = "Easting", ylab = "Northing"
+        )
         plotList <- append(plotList, list(p))
       }
     }
-    if(is.null(ncol)){ncol <- round(sqrt(length(r@layers)))}
-    do.call(eval(parse(text="gridExtra::grid.arrange")), c(plotList, ncol = ncol))
+    if (is.null(ncol)) {
+      ncol <- round(sqrt(length(r@layers)))
+    }
+    do.call(eval(parse(text = "gridExtra::grid.arrange")), c(plotList, ncol = ncol))
   }
 }
 
 #' Converts a rasterStack to logarithmic scale
-#' 
+#'
 #' Avoids the problem of -Inf occuring for log(0).
 #'
 #' @param rStack rasterStack to convert to logarithmic scale
@@ -167,17 +178,16 @@ plotRaster <- function(r, title = character(0), centerColorBar = FALSE, ncol = N
 #'
 #' @examples
 #' logRasterStack(raster::stack(dem))
-logRasterStack <- function(rStack, standartize = FALSE, InfVal = NA)
-{
+logRasterStack <- function(rStack, standartize = FALSE, InfVal = NA) {
   rStack <- log(rStack)
   rStack[is.infinite(rStack)] <- InfVal
-  if(standartize) {
+  if (standartize) {
     naInd <- is.na(rStack)
     rStack[naInd] <- 0
     maxR <- max(raster::maxValue(rStack))
     minR <- min(raster::minValue(rStack))
     rStack <- raster::stack(rStack)
-    for(i in 1:length(rStack@layers)) {
+    for (i in seq_len(length(rStack@layers))) {
       rStack@layers[[i]] <- (rStack@layers[[i]] - minR) / (maxR - minR)
     }
     rStack[naInd] <- NA
@@ -186,7 +196,7 @@ logRasterStack <- function(rStack, standartize = FALSE, InfVal = NA)
 }
 
 #' Export a dataCube as image slice sequence
-#' 
+#'
 #' Exports a dataCube of type \code{rasterStack} as Tiff image sequence.
 #' Image sequences are a common structure to represent voxel data and
 #' most of the specific software to visualize voxel data is able to read it (e.g. blender)
@@ -200,20 +210,23 @@ logRasterStack <- function(rStack, standartize = FALSE, InfVal = NA)
 #' @export
 #'
 #' @examples
-#' crws <- lapply(X=seq(1:100), FUN = function(X) {
+#' crws <- lapply(X = seq(1:100), FUN = function(X) {
 #'   sim.crw.3d(nStep = 100, rTurn = 0.99, rLift = 0.99, meanStep = 0.1)
 #' })
 #' points <- do.call("rbind", crws)
 #' extent <- raster::extent(c(-10, 10, -10, 10))
-#' ud <- voxelCount(points, extent, xyRes=5,
-#'                  zMin=-10, zMax=10, standartize = TRUE)
+#' ud <- voxelCount(points, extent,
+#'   xyRes = 5,
+#'   zMin = -10, zMax = 10, standartize = TRUE
+#' )
 #' saveImageSlices(ud, filename = "saveImageSlices_test", dir = tempdir())
-saveImageSlices <- function(rStack, filename, dir, NaVal = 0)
-{
+saveImageSlices <- function(rStack, filename, dir, NaVal = 0) {
   rStack[is.na(rStack)] <- NaVal
   rStack <- raster::stack(rStack)
-  for (i in 1:length(rStack@layers)) {
-    tiff::writeTIFF(raster::as.matrix(rStack[[i]]),
-                    file.path(dir, paste(sprintf("%03d", i), ".", filename, ".tif", sep = "")))
+  for (i in seq_len(length(rStack@layers))) {
+    tiff::writeTIFF(
+      raster::as.matrix(rStack[[i]]),
+      file.path(dir, paste(sprintf("%03d", i), ".", filename, ".tif", sep = ""))
+    )
   }
 }
