@@ -23,21 +23,36 @@
 #'
 #' @examples
 #' get.track.densities.3d(niclas, heightDist = TRUE)
-get.track.densities.3d <- function(track, gradientDensity = TRUE, heightDistEllipsoid = TRUE, DEM = NULL, maxBin = 25)
-{
+get.track.densities.3d <- function(track, gradientDensity = TRUE, heightDistEllipsoid = TRUE, DEM = NULL, maxBin = 25) {
   .is.df.xyz(track)
   track <- track.properties.3d(track)
-  turnAngle <- track$t[2:nrow(track)]; liftAngle <- track$l[2:nrow(track)]; stepLength <- track$d[2:nrow(track)]
-  deltaTurn <- diff(turnAngle); deltaLift <- diff(liftAngle); deltaStep <- diff(stepLength)
-  if (gradientDensity) {gradientAngle <- track$g} else {gradientAngle <- NULL}
-  if (heightDistEllipsoid) {heightEllipsoid <- track$z} else {heightEllipsoid <- NULL}
+  turnAngle <- track$t[2:nrow(track)]
+  liftAngle <- track$l[2:nrow(track)]
+  stepLength <- track$d[2:nrow(track)]
+  deltaTurn <- diff(turnAngle)
+  deltaLift <- diff(liftAngle)
+  deltaStep <- diff(stepLength)
+  if (gradientDensity) {
+    gradientAngle <- track$g
+  } else {
+    gradientAngle <- NULL
+  }
+  if (heightDistEllipsoid) {
+    heightEllipsoid <- track$z
+  } else {
+    heightEllipsoid <- NULL
+  }
   if (!is.null(DEM)) {
     .check.extent(DEM = DEM, track = track)
-    heightTopo <- track$z - raster::extract(DEM, track[,1:2])
-  } else {heightTopo <- NULL}
-  return(get.densities.3d(turnAngle = turnAngle, liftAngle = liftAngle, stepLength = stepLength,
-                          deltaLift = deltaLift, deltaTurn = deltaTurn, deltaStep = deltaStep, gradientAngle = gradientAngle,
-                          heightEllipsoid = heightEllipsoid, heightTopo = heightTopo, maxBin = maxBin))
+    heightTopo <- track$z - raster::extract(DEM, track[, 1:2])
+  } else {
+    heightTopo <- NULL
+  }
+  return(get.densities.3d(
+    turnAngle = turnAngle, liftAngle = liftAngle, stepLength = stepLength,
+    deltaLift = deltaLift, deltaTurn = deltaTurn, deltaStep = deltaStep, gradientAngle = gradientAngle,
+    heightEllipsoid = heightEllipsoid, heightTopo = heightTopo, maxBin = maxBin
+  ))
 }
 
 #' Extract tldCube and autodifferences functions from track sections
@@ -59,23 +74,36 @@ get.track.densities.3d <- function(track, gradientDensity = TRUE, heightDistElli
 #'
 #' @examples
 #' get.section.densities.3d(list(niclas[1:10, ], niclas[11:nrow(niclas), ]))
-get.section.densities.3d <- function(trackSections, gradientDensity = TRUE, heightDistEllipsoid = TRUE, DEM = NULL, maxBin = 25)
-{
-  trackSections <- lapply(X=trackSections, FUN= function(X) track.properties.3d(X)[2:nrow(X), ])
+get.section.densities.3d <- function(trackSections, gradientDensity = TRUE, heightDistEllipsoid = TRUE, DEM = NULL, maxBin = 25) {
+  trackSections <- lapply(X = trackSections, FUN = function(X) track.properties.3d(X)[2:nrow(X), ])
   deltaTurn <- Reduce(c, lapply(X = trackSections, FUN = function(X) diff(X$t)))
   deltaLift <- Reduce(c, lapply(X = trackSections, FUN = function(X) diff(X$l)))
   deltaStep <- Reduce(c, lapply(X = trackSections, FUN = function(X) diff(X$d)))
   trackSections <- do.call(rbind, trackSections)
-  turnAngle <- trackSections$t; liftAngle <- trackSections$l; stepLength <- trackSections$d
-  if (gradientDensity) {gradientAngle <- trackSections$g} else {gradientAngle <- NULL}
-  if (heightDistEllipsoid) {heightEllipsoid <- trackSections$z} else {heightEllipsoid <- NULL}
+  turnAngle <- trackSections$t
+  liftAngle <- trackSections$l
+  stepLength <- trackSections$d
+  if (gradientDensity) {
+    gradientAngle <- trackSections$g
+  } else {
+    gradientAngle <- NULL
+  }
+  if (heightDistEllipsoid) {
+    heightEllipsoid <- trackSections$z
+  } else {
+    heightEllipsoid <- NULL
+  }
   if (!is.null(DEM)) {
     .check.extent(DEM = DEM, track = trackSections)
-    heightTopo <- trackSections$z - raster::extract(DEM, trackSections[,1:2])
-  } else {heightTopo <- NULL}
-  return(get.densities.3d(turnAngle = turnAngle, liftAngle = liftAngle, stepLength = stepLength,
-                          deltaLift = deltaLift, deltaTurn = deltaTurn, deltaStep = deltaStep, gradientAngle = gradientAngle,
-                          heightEllipsoid = heightEllipsoid, heightTopo = heightTopo, maxBin = maxBin))
+    heightTopo <- trackSections$z - raster::extract(DEM, trackSections[, 1:2])
+  } else {
+    heightTopo <- NULL
+  }
+  return(get.densities.3d(
+    turnAngle = turnAngle, liftAngle = liftAngle, stepLength = stepLength,
+    deltaLift = deltaLift, deltaTurn = deltaTurn, deltaStep = deltaStep, gradientAngle = gradientAngle,
+    heightEllipsoid = heightEllipsoid, heightTopo = heightTopo, maxBin = maxBin
+  ))
 }
 
 #' This function splits the by outliers in the time lag.
@@ -92,22 +120,31 @@ get.section.densities.3d <- function(trackSections, gradientDensity = TRUE, heig
 #' @export
 #'
 #' @examples
-#' track.split.3d(niclas, timeLag=rep(1, nrow(niclas)-1) + rnorm(nrow(niclas)-1, mean = 0, sd = 0.25))
-track.split.3d <- function(track, timeLag, lag = NULL, tolerance = NULL)
-{
+#' track.split.3d(
+#'   niclas,
+#'   timeLag = rep(1, nrow(niclas) - 1) + rnorm(nrow(niclas) - 1,
+#'   mean = 0,
+#'   sd = 0.25)
+#' )
+track.split.3d <- function(track, timeLag, lag = NULL, tolerance = NULL) {
   .is.df.xyz(track)
   if (any(is.na(timeLag))) stop("TimeLag is not allowed to contain NAs.")
   if (is.null(lag)) {
     m <- mean(timeLag)
-  } else {m <- lag}
+  } else {
+    m <- lag
+  }
   if (is.null(tolerance)) {
     tolerance <- 0.5 * stats::sd(timeLag)
   }
-  splitRows <- which(abs(m-timeLag) > tolerance)
-  trackSections <- split(track, cumsum(1:nrow(track) %in% (splitRows+2))) # + 1 if the cut should be one step before
-  nSplits <- length(splitRows); nChange <- round(sum(timeLag[splitRows]/m-1));
-  message(paste("  |Mean time lag: ", round(m,2) , ", tolerance: ", round(tolerance,2),
-                ", number of splits: ", nSplits, ", proposed change in steps: ", nChange, sep=""))
+  splitRows <- which(abs(m - timeLag) > tolerance)
+  trackSections <- split(track, cumsum(seq_len(nrow(track)) %in% (splitRows + 2))) # + 1 if the cut should be one step before
+  nSplits <- length(splitRows)
+  nChange <- round(sum(timeLag[splitRows] / m - 1))
+  message(paste("  |Mean time lag: ", round(m, 2), ", tolerance: ", round(tolerance, 2),
+    ", number of splits: ", nSplits, ", proposed change in steps: ", nChange,
+    sep = ""
+  ))
   return(trackSections)
 }
 
@@ -122,11 +159,10 @@ track.split.3d <- function(track, timeLag, lag = NULL, tolerance = NULL)
 #'
 #' @examples
 #' dem2track.extent(dem, niclas)
-dem2track.extent <- function(DEM, track, buffer=100)
-{
+dem2track.extent <- function(DEM, track, buffer = 100) {
   .is.df.xyz(track = track)
   .check.extent(DEM = DEM, track = track)
-  return(raster::crop(DEM, raster::extent(min(track$x)-buffer, max(track$x)+buffer, min(track$y)-buffer, max(track$y)+buffer)))
+  return(raster::crop(DEM, raster::extent(min(track$x) - buffer, max(track$x) + buffer, min(track$y) - buffer, max(track$y) + buffer)))
 }
 
 #' Extent of track(s)
@@ -140,18 +176,27 @@ dem2track.extent <- function(DEM, track, buffer=100)
 #'
 #' @examples
 #' track.extent(niclas, zAxis = TRUE)
-track.extent <- function(track, zAxis = FALSE){
+track.extent <- function(track, zAxis = FALSE) {
   if (!is.list(track) || !is.list(track)) stop("Track input has to be of type list or data.frame.")
-  if (is.list(track) && is.data.frame(track)) {track <- list(track)}
-  extents <- do.call("rbind", lapply(X = track, FUN = function(track){
-    c(floor(min(track$x)), floor(max(track$x))+1,
-      floor(min(track$y)), floor(max(track$y))+1,
-      floor(min(track$z)), floor(max(track$z))+1)
+  if (is.list(track) && is.data.frame(track)) {
+    track <- list(track)
+  }
+  extents <- do.call("rbind", lapply(X = track, FUN = function(track) {
+    c(
+      floor(min(track$x)), floor(max(track$x)) + 1,
+      floor(min(track$y)), floor(max(track$y)) + 1,
+      floor(min(track$z)), floor(max(track$z)) + 1
+    )
   }))
-  minX <- min(extents[,1]); maxX <- max(extents[,2]);
-  minY <- min(extents[,3]); maxY <- max(extents[,4]);
-  minZ <- min(extents[,5]); maxZ <- max(extents[,6]);
-  if (zAxis) {return(rbind(xmin=minX, xmax=maxX, ymin=minY, ymax=maxY, zmin=minZ, zmax=maxZ))}
+  minX <- min(extents[, 1])
+  maxX <- max(extents[, 2])
+  minY <- min(extents[, 3])
+  maxY <- max(extents[, 4])
+  minZ <- min(extents[, 5])
+  maxZ <- max(extents[, 6])
+  if (zAxis) {
+    return(rbind(xmin = minX, xmax = maxX, ymin = minY, ymax = maxY, zmin = minZ, zmax = maxZ))
+  }
   return(raster::extent(minX, maxX, minY, maxY))
 }
 
@@ -165,11 +210,10 @@ track.extent <- function(track, zAxis = FALSE){
 #' @examples
 #' .is.df.xyz(niclas)
 #' @noRd
-.is.df.xyz <- function(track)
-{
-  if(!class(track)=="data.frame") stop("Input not of type 'data.frame'.")
-  if(!any(colnames(track)[1:3]==c("x","y","z"))) stop("Colnames of first three cols not 'x, y, z'.")
-  if(any(is.na(track[, 1:3]))) stop("Track 'data.frame' contains NA values.")
+.is.df.xyz <- function(track) {
+  if (!class(track) == "data.frame") stop("Input not of type 'data.frame'.")
+  if (!any(colnames(track)[1:3] == c("x", "y", "z"))) stop("Colnames of first three cols not 'x, y, z'.")
+  if (any(is.na(track[, 1:3]))) stop("Track 'data.frame' contains NA values.")
 }
 
 #' Checks if the track lies inside the digital elevation model.
@@ -183,12 +227,13 @@ track.extent <- function(track, zAxis = FALSE){
 #' @examples
 #' .check.extent(dem, niclas)
 #' @noRd
-.check.extent <- function(DEM, track)
-{
-  if(!class(DEM)=="RasterLayer") stop("'DEM' is not of type 'RasterLayer'")
+.check.extent <- function(DEM, track) {
+  if (!class(DEM) == "RasterLayer") stop("'DEM' is not of type 'RasterLayer'")
   e <- raster::extent(DEM)
-  if(!(min(track[,1]) >= e[1] && max(track[,1]) <= e[2] &&
-       min(track[,2]) >= e[3] && max(track[,2]) <= e[4])) stop("The track is not inside the area of the digital elevation model.")
+  if (!(min(track[, 1]) >= e[1] && max(track[, 1]) <= e[2] &&
+    min(track[, 2]) >= e[3] && max(track[, 2]) <= e[4])) {
+    stop("The track is not inside the area of the digital elevation model.")
+  }
 }
 
 #' Reproduce a track with the eRTG3D
@@ -213,31 +258,52 @@ track.extent <- function(track, zAxis = FALSE){
 #'
 #' @examples
 #' reproduce.track.3d(niclas[1:10, ])
-reproduce.track.3d <- function(track, n.sim = 1, parallel = FALSE, error = TRUE, DEM = NULL, BG = NULL, filterDeadEnds = TRUE, plot2d = FALSE, plot3d = FALSE, maxBin = 25, gradientDensity = TRUE)
-{
+reproduce.track.3d <- function(track, n.sim = 1, parallel = FALSE, error = TRUE, DEM = NULL, BG = NULL, filterDeadEnds = TRUE, plot2d = FALSE, plot3d = FALSE, maxBin = 25, gradientDensity = TRUE) {
   .is.df.xyz(track = track)
   track <- track.properties.3d(track)
   n.locs <- nrow(track)
   if (n.locs > 1500) message("  |Note: The track is very long (>1500 steps), the system may run out of memory.")
-  turnAngle <- track$t[2:nrow(track)]; liftAngle <- track$l[2:nrow(track)]; stepLength <- track$d[2:nrow(track)]
-  deltaTurn <- diff(turnAngle); deltaLift <- diff(liftAngle); deltaStep <- diff(stepLength)
+  turnAngle <- track$t[2:nrow(track)]
+  liftAngle <- track$l[2:nrow(track)]
+  stepLength <- track$d[2:nrow(track)]
+  deltaTurn <- diff(turnAngle)
+  deltaLift <- diff(liftAngle)
+  deltaStep <- diff(stepLength)
   heightEllipsoid <- track$z
-  if (gradientDensity) {gradientAngle <- track$g} else {gradientAngle <- NULL}
+  if (gradientDensity) {
+    gradientAngle <- track$g
+  } else {
+    gradientAngle <- NULL
+  }
   if (!is.null(DEM)) {
     .check.extent(DEM = DEM, track = track)
-    heightTopo <- track$z - raster::extract(DEM, track[,1:2])
-  } else {heightTopo <- NULL}
-  D <- get.densities.3d(liftAngle = liftAngle, turnAngle = turnAngle, stepLength = stepLength,
-                        deltaLift = deltaLift, deltaTurn = deltaTurn, deltaStep = deltaStep, gradientAngle = gradientAngle,
-                        heightEllipsoid = heightEllipsoid, heightTopo = heightTopo, maxBin = maxBin)
-  uerw <- sim.uncond.3d(n.locs*1500, start = c(track$x[1],track$y[1],track$z[1]),
-                        a0 = track$a[1], g0 = track$g[1], densities = D, error = error)
+    heightTopo <- track$z - raster::extract(DEM, track[, 1:2])
+  } else {
+    heightTopo <- NULL
+  }
+  D <- get.densities.3d(
+    liftAngle = liftAngle, turnAngle = turnAngle, stepLength = stepLength,
+    deltaLift = deltaLift, deltaTurn = deltaTurn, deltaStep = deltaStep, gradientAngle = gradientAngle,
+    heightEllipsoid = heightEllipsoid, heightTopo = heightTopo, maxBin = maxBin
+  )
+  uerw <- sim.uncond.3d(n.locs * 1500,
+    start = c(track$x[1], track$y[1], track$z[1]),
+    a0 = track$a[1], g0 = track$g[1], densities = D, error = error
+  )
   Q <- qProb.3d(uerw, n.locs, parallel = parallel, maxBin = maxBin)
-  cerwList <- suppressWarnings(n.sim.cond.3d(n.sim = n.sim, n.locs <- n.locs, start=c(track$x[1],track$y[1],track$z[1]), end=c(track$x[n.locs],track$y[n.locs],track$z[n.locs]),
-                                             a0 = track$a[1], g0 = track$g[1], densities=D, qProbs=Q, error = error, parallel = parallel, DEM = DEM, BG = BG))
-  if(filterDeadEnds){cerwList <- filter.dead.ends(cerwList)}
-  if(plot2d){print(plot2d(origTrack = track, simTrack = cerwList, DEM = DEM))}
-  if(plot3d){plot3d(origTrack = track, simTrack = cerwList, DEM = DEM)}
+  cerwList <- suppressWarnings(n.sim.cond.3d(
+    n.sim = n.sim, n.locs <- n.locs, start = c(track$x[1], track$y[1], track$z[1]), end = c(track$x[n.locs], track$y[n.locs], track$z[n.locs]),
+    a0 = track$a[1], g0 = track$g[1], densities = D, qProbs = Q, error = error, parallel = parallel, DEM = DEM, BG = BG
+  ))
+  if (filterDeadEnds) {
+    cerwList <- filter.dead.ends(cerwList)
+  }
+  if (plot2d) {
+    print(plot2d(origTrack = track, simTrack = cerwList, DEM = DEM))
+  }
+  if (plot3d) {
+    plot3d(origTrack = track, simTrack = cerwList, DEM = DEM)
+  }
   return(cerwList)
 }
 
@@ -252,14 +318,21 @@ reproduce.track.3d <- function(track, n.sim = 1, parallel = FALSE, error = TRUE,
 #'
 #' @examples
 #' filter.dead.ends(list(niclas, niclas))
-filter.dead.ends <- function(cerwList)
-{
-  if(is.null(cerwList)) {warning("No track made it to the end point."); return(NULL)}
+filter.dead.ends <- function(cerwList) {
+  if (is.null(cerwList)) {
+    warning("No track made it to the end point.")
+    return(NULL)
+  }
   l1 <- length(cerwList)
   cerwList <- cerwList[!unlist(lapply(cerwList, is.null))]
   l2 <- length(cerwList)
-  if (l2 == 0) {warning("No track made it to the end point."); return(NULL)}
-  if (l1!=l2) {message(paste("  |Dead end tracks removed (n = ", (l1-l2), ", proportion: ", 1-round(l2/l1, 2), ")", sep=""))}
+  if (l2 == 0) {
+    warning("No track made it to the end point.")
+    return(NULL)
+  }
+  if (l1 != l2) {
+    message(paste("  |Dead end tracks removed (n = ", (l1 - l2), ", proportion: ", 1 - round(l2 / l1, 2), ")", sep = ""))
+  }
   return(cerwList)
 }
 
@@ -280,15 +353,19 @@ filter.dead.ends <- function(cerwList)
 #'
 #' @examples
 #' movingMedian(sequence(1:10), window = 5)
-movingMedian <- function(data, window){
-  if(!(window %% 2 == 0)) {window <- floor(window/2)} else{stop("Window must be an uneven number.")}
+movingMedian <- function(data, window) {
+  if (!(window %% 2 == 0)) {
+    window <- floor(window / 2)
+  } else {
+    stop("Window must be an uneven number.")
+  }
   total <- length(data)
   result <- vector(length = total)
-  for(i in (window+1):(total-window)){
-    result[i] <- stats::median(data[(i-window):(i+window)])
+  for (i in (window + 1):(total - window)) {
+    result[i] <- stats::median(data[(i - window):(i + window)])
   }
   result[1:window] <- stats::median(data[1:window])
-  result[(total-window):total] <- stats::median(data[(total-window):total])
+  result[(total - window):total] <- stats::median(data[(total - window):total])
   return(result)
 }
 
@@ -307,7 +384,8 @@ turn2target.3d <- function(track) {
   .is.df.xyz(track = track)
   track <- track.properties.3d(track)
   target <- Reduce(c, track[nrow(track), 1:3])
-  .wrap(atan2(target[2]-track$y, target[1] - track$x) - track$a)}
+  .wrap(atan2(target[2] - track$y, target[1] - track$x) - track$a)
+}
 
 #' Lift angle to target
 #'
@@ -324,8 +402,11 @@ lift2target.3d <- function(track) {
   .is.df.xyz(track = track)
   track <- track.properties.3d(track)
   target <- Reduce(c, track[nrow(track), 1:3])
-  .wrap(atan2(sqrt((target[1]-track$x) ^ 2 + (target[2]-track$y) ^ 2),
-              (target[3]-track$z)) - track$g)}
+  .wrap(atan2(
+    sqrt((target[1] - track$x)^2 + (target[2] - track$y)^2),
+    (target[3] - track$z)
+  ) - track$g)
+}
 
 #' Distance to target
 #'
@@ -341,7 +422,8 @@ lift2target.3d <- function(track) {
 dist2target.3d <- function(track) {
   .is.df.xyz(track = track)
   target <- Reduce(c, track[nrow(track), 1:3])
-  sqrt((target[1]-track$x) ^ 2 + (target[2]-track$y) ^ 2 + (target[3]-track$z) ^ 2)}
+  sqrt((target[1] - track$x)^2 + (target[2] - track$y)^2 + (target[3] - track$z)^2)
+}
 
 #' Distance of each track point to a given point
 #'
@@ -353,13 +435,13 @@ dist2target.3d <- function(track) {
 #' @export
 #'
 #' @examples
-#' dist2point.3d(niclas, c(0,0,0))
+#' dist2point.3d(niclas, c(0, 0, 0))
 dist2point.3d <- function(track, point, groundDistance = FALSE) {
   .is.df.xyz(track = track)
-  if (groundDistance | length(point)==2) {
-    sqrt((point[1]-track[,1])^2 + (point[2]-track[,2])^2)
+  if (groundDistance | length(point) == 2) {
+    sqrt((point[1] - track[, 1])^2 + (point[2] - track[, 2])^2)
   } else {
-    sqrt((point[1]-track[,1])^2 + (point[2]-track[,2])^2 + (point[3]-track[,3])^2)
+    sqrt((point[1] - track[, 1])^2 + (point[2] - track[, 2])^2 + (point[3] - track[, 3])^2)
   }
 }
 
@@ -375,21 +457,24 @@ dist2point.3d <- function(track, point, groundDistance = FALSE) {
 #'
 #' @examples
 #' track.properties.3d(niclas)
-track.properties.3d <- function(track)
-{
+track.properties.3d <- function(track) {
   .is.df.xyz(track)
   # spatial coordinates
-  x <- track[,1]; y <- track[,2]; z <- track[,3]
+  x <- track[, 1]
+  y <- track[, 2]
+  z <- track[, 3]
   # distance covered per axis and step
-  dx <- c(NA, diff(x)); dy <- c(NA, diff(y)); dz <- c(NA, diff(z))
+  dx <- c(NA, diff(x))
+  dy <- c(NA, diff(y))
+  dz <- c(NA, diff(z))
   d2d <- .distance.2d(dx, dy)
   # spherical coordinates
   d <- .distance.3d(dx, dy, dz)
   a <- .get.azimut(dx, dy)
   g <- .get.polar(d2d, dz)
   # guess the initial azimut/heading & polar angle/gradient (alt: sample(a,1))
-  a[1] <- mean(a, na.rm=TRUE)
-  g[1] <- mean(g, na.rm=TRUE)
+  a[1] <- mean(a, na.rm = TRUE)
+  g[1] <- mean(g, na.rm = TRUE)
   # Turn angle and lift angle
   t <- c(NA, diff(a))
   l <- c(NA, diff(g))
@@ -407,8 +492,7 @@ track.properties.3d <- function(track)
 #' @examples
 #' .get.azimut(10, 30)
 #' @noRd
-.get.azimut <- function(dx, dy)
-{
+.get.azimut <- function(dx, dy) {
   .wrap(atan2(dy, dx))
 }
 
@@ -423,8 +507,7 @@ track.properties.3d <- function(track)
 #' @examples
 #' .get.polar(10, 50)
 #' @noRd
-.get.polar <- function(d, dz)
-{
+.get.polar <- function(d, dz) {
   .wrap(atan2(d, dz))
 }
 
@@ -439,9 +522,8 @@ track.properties.3d <- function(track)
 #' @examples
 #' .distance.2d(10, 30)
 #' @noRd
-.distance.2d <- function(dx, dy)
-{
-  sqrt(dx*dx+dy*dy)
+.distance.2d <- function(dx, dy) {
+  sqrt(dx * dx + dy * dy)
 }
 
 #' Distance covered in 3 dimensions, radius
@@ -456,7 +538,6 @@ track.properties.3d <- function(track)
 #' @examples
 #' .distance.3d(10, 10, 10)
 #' @noRd
-.distance.3d <- function(dx, dy, dz)
-{
-  sqrt(dx*dx+dy*dy+dz*dz)
+.distance.3d <- function(dx, dy, dz) {
+  sqrt(dx * dx + dy * dy + dz * dz)
 }
