@@ -1,10 +1,10 @@
 test_that("sf linkage functions work", {
   # To sf
-  niclas.sf <- track2sf.3d(niclas, CRS = "+init=epsg:2056")
+  niclas.sf <- track2sf.3d(niclas, CRS = 2056)
   expect_equal(is.sf.3d(niclas.sf), TRUE)
   expect_equal(nrow(niclas), nrow(niclas.sf))
   expect_is(niclas.sf$geometry, c("sfc_POINT", "sfc"))
-  niclas.sf.m <- track2sf.3d(as.matrix(niclas), CRS = "+init=epsg:2056")
+  niclas.sf.m <- track2sf.3d(as.matrix(niclas), CRS = 2056)
   expect_equal(is.sf.3d(niclas.sf.m), TRUE)
   expect_equal(nrow(niclas), nrow(niclas.sf.m))
   expect_is(niclas.sf.m$geometry, c("sfc_POINT", "sfc"))
@@ -15,8 +15,11 @@ test_that("sf linkage functions work", {
   expect_equal(is.sf.3d(niclas.df.m), FALSE)
   # transform CRS
   row.names(niclas) <- NULL
-  expect_equal(niclas,
-               transformCRS.3d(
-                 transformCRS.3d(niclas, fromCRS="+init=epsg:2056", toCRS="+init=epsg:4326"),
-                 toCRS="+init=epsg:2056", fromCRS="+init=epsg:4326")[, 1:3])
+  expect_equal(
+    niclas,
+    transformCRS.3d(
+      transformCRS.3d(niclas, fromCRS = 2056, toCRS = 4326),
+      toCRS = 2056, fromCRS = 4326
+    )[, 1:3]
+  )
 })
